"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var global_state_1 = require("./global-state");
var observer_1 = require("./observer");
var utils_1 = require("./utils");
var Atom = (function () {
    function Atom(onBecomeObservedHandler, onBecomeUnobservedHandler) {
        if (onBecomeObservedHandler === void 0) { onBecomeObservedHandler = utils_1.noop; }
        if (onBecomeUnobservedHandler === void 0) { onBecomeUnobservedHandler = utils_1.noop; }
        /**
         * All reactions with this atom.
         */
        this.reactions = new Set();
        this.isBeingTracked = false;
        this.onBecomeObservedHandler = onBecomeObservedHandler;
        this.onBecomeUnobservedHandler = onBecomeUnobservedHandler;
    }
    /**
     * Report observe, will trigger onBecomeObservedHandler in the first time.
     */
    Atom.prototype.reportObserved = function () {
        observer_1.startBatch();
        if (global_state_1.globalState.currentReaction) {
            // Binding current reaction
            this.reactions.add(global_state_1.globalState.currentReaction);
        }
        if (!this.isBeingTracked) {
            this.isBeingTracked = true;
            this.onBecomeObservedHandler();
        }
        observer_1.endBatch();
    };
    /**
     * report changed, so will trigger all reactions.
     */
    Atom.prototype.reportChanged = function () {
        this.reactions.forEach(function (reaction) {
            observer_1.runReactionAsync(reaction);
        });
    };
    Atom.prototype.unobserve = function () {
        this.isBeingTracked = false;
        this.reactions.clear();
        this.onBecomeUnobservedHandler();
    };
    return Atom;
}());
exports.Atom = Atom;
//# sourceMappingURL=atom.js.map