"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var global_state_1 = require("../global-state");
var native = Map.prototype;
var masterKey = Symbol("Map master key");
var getters = ["has", "get"];
var iterators = ["forEach", "keys", "values", "entries", Symbol.iterator];
var all = ["set", "delete", "clear"].concat(getters, iterators);
function shim(target, bindCurrentReaction, queueRunReactions, proxyValue) {
    target.$raw = {};
    var _loop_1 = function (method) {
        // tslint:disable-next-line:space-before-function-paren only-arrow-functions
        target.$raw[method] = function () {
            native[method].apply(target, arguments);
        };
    };
    for (var _i = 0, all_1 = all; _i < all_1.length; _i++) {
        var method = all_1[_i];
        _loop_1(method);
    }
    var _loop_2 = function (getter) {
        // tslint:disable-next-line:space-before-function-paren only-arrow-functions
        target[getter] = function (key) {
            var value = native[getter].apply(this, arguments);
            global_state_1.globalState.event.emit("get", { target: target, key: key, value: value });
            value = proxyValue(this, key, value);
            bindCurrentReaction(this, key);
            return value;
        };
    };
    for (var _a = 0, getters_1 = getters; _a < getters_1.length; _a++) {
        var getter = getters_1[_a];
        _loop_2(getter);
    }
    var _loop_3 = function (iterator) {
        // tslint:disable-next-line:space-before-function-paren only-arrow-functions
        target[iterator] = function () {
            bindCurrentReaction(this, masterKey);
            return native[iterator].apply(this, arguments);
        };
    };
    for (var _b = 0, iterators_1 = iterators; _b < iterators_1.length; _b++) {
        var iterator = iterators_1[_b];
        _loop_3(iterator);
    }
    // tslint:disable-next-line:space-before-function-paren only-arrow-functions
    target.set = function (key, value) {
        var oldValue = this.get(key);
        var result = native.set.apply(this, arguments);
        global_state_1.globalState.event.emit("set", { target: target, key: key, oldValue: oldValue, value: value });
        if (oldValue !== value) {
            queueRunReactions(this, key);
            queueRunReactions(this, masterKey);
        }
        return result;
    };
    // tslint:disable-next-line:space-before-function-paren only-arrow-functions
    target.delete = function (key) {
        var has = this.has(key);
        var result = native.delete.apply(this, arguments);
        global_state_1.globalState.event.emit("deleteProperty", { target: target, key: key });
        if (has) {
            queueRunReactions(this, key);
            queueRunReactions(this, masterKey);
        }
        return result;
    };
    // tslint:disable-next-line:space-before-function-paren only-arrow-functions
    target.clear = function () {
        var size = this.size;
        var result = native.clear.apply(this, arguments);
        if (size) {
            queueRunReactions(this, masterKey);
        }
        return result;
    };
    Object.defineProperty(target, "size", {
        get: function get() {
            var proto = Object.getPrototypeOf(this);
            var size = Reflect.get(proto, "size", this);
            bindCurrentReaction(this, masterKey);
            return size;
        }
    });
    return target;
}
exports.default = shim;
//# sourceMappingURL=map.js.map