"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var global_state_1 = require("../global-state");
var native = WeakMap.prototype;
var getters = ["has", "get"];
var all = ["set", "delete"].concat(getters);
function shim(target, bindCurrentReaction, queueRunReactions, proxyValue) {
    target.$raw = {};
    var _loop_1 = function (method) {
        // tslint:disable-next-line:space-before-function-paren only-arrow-functions
        target.$raw[method] = function () {
            native[method].apply(target, arguments);
        };
    };
    for (var _i = 0, all_1 = all; _i < all_1.length; _i++) {
        var method = all_1[_i];
        _loop_1(method);
    }
    var _loop_2 = function (getter) {
        // tslint:disable-next-line:space-before-function-paren only-arrow-functions
        target[getter] = function (key) {
            var value = native[getter].apply(this, arguments);
            global_state_1.globalState.event.emit("get", { target: target, key: null, value: value });
            value = proxyValue(this, key, value);
            bindCurrentReaction(this, key);
            return value;
        };
    };
    for (var _a = 0, getters_1 = getters; _a < getters_1.length; _a++) {
        var getter = getters_1[_a];
        _loop_2(getter);
    }
    // tslint:disable-next-line:space-before-function-paren only-arrow-functions
    target.set = function (key, value) {
        var oldValue = this.get(key);
        var result = native.set.apply(this, arguments);
        global_state_1.globalState.event.emit("set", { target: target, key: key, oldValue: oldValue, value: value });
        if (oldValue !== value) {
            queueRunReactions(this, key);
        }
        return result;
    };
    // tslint:disable-next-line:space-before-function-paren only-arrow-functions
    target.delete = function (key) {
        var has = this.has(key);
        var result = native.delete.apply(this, arguments);
        global_state_1.globalState.event.emit("deleteProperty", { target: target, key: key });
        if (has) {
            queueRunReactions(this, key);
        }
        return result;
    };
    return target;
}
exports.default = shim;
//# sourceMappingURL=weak-map.js.map