"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var global_state_1 = require("./global-state");
var utils_1 = require("./utils");
// tslint:disable-next-line:no-var-requires
var cloneDeep = require("lodash.clonedeep");
function reportChange(change) {
    if (global_state_1.globalState.currentDebugOutputAction) {
        global_state_1.globalState.currentDebugOutputAction.changeList.push(change);
    }
    else {
        global_state_1.globalState.event.emit("debug", {
            id: utils_1.createUniqueId(),
            name: null,
            changeList: [change],
            type: "isolated"
        });
    }
}
function getCallStack(target) {
    var callStack = [];
    if (!global_state_1.globalState.parentInfo.has(target)) {
        callStack.unshift(target.constructor.name);
    }
    else {
        var currentTarget = target;
        var runCount = 0;
        while (global_state_1.globalState.parentInfo.has(currentTarget)) {
            var parentInfo = global_state_1.globalState.parentInfo.get(currentTarget);
            // add key to call stack
            callStack.unshift(parentInfo.key);
            // If has no parent's parent, add parent's name to call stack.
            if (!global_state_1.globalState.parentInfo.has(parentInfo.parent)) {
                callStack.unshift(parentInfo.parent.constructor.name);
            }
            currentTarget = parentInfo.parent;
            runCount++;
            if (runCount >= 50) {
                break;
            }
        }
    }
    return callStack;
}
/**
 * Store parentInfo when get triggered.
 */
global_state_1.globalState.event.on("get", function (info) {
    if (!global_state_1.globalState.useDebug) {
        return;
    }
    if (info.value !== null && typeof info.value === "object") {
        global_state_1.globalState.parentInfo.set(info.value, {
            parent: info.target,
            key: info.key
        });
    }
});
/**
 * Snapshot current Action's debugName, when runInAction triggered.
 */
global_state_1.globalState.event.on("runInAction", function (debugName) {
    if (!global_state_1.globalState.useDebug) {
        return;
    }
    global_state_1.globalState.currentDebugName = debugName || null;
});
/**
 * debug 入栈 action
 */
global_state_1.globalState.event.on("startBatch", function () {
    if (!global_state_1.globalState.useDebug) {
        return;
    }
    var debugOutputBundleAction = {
        name: global_state_1.globalState.currentDebugName,
        changeList: [],
        type: "action"
    };
    global_state_1.globalState.debugOutputActionMapBatchDeep.set(global_state_1.globalState.batchDeep, debugOutputBundleAction);
    global_state_1.globalState.currentDebugOutputAction = debugOutputBundleAction;
    // 如果深度等于 1，生成唯一 id 给这个 action
    if (global_state_1.globalState.batchDeep === 1) {
        debugOutputBundleAction.id = utils_1.createUniqueId();
        global_state_1.globalState.currentDebugId = debugOutputBundleAction.id;
    }
    // 如果当前深度大于 1，就作为加到父级的 changeList
    if (global_state_1.globalState.batchDeep > 1) {
        global_state_1.globalState.debugOutputActionMapBatchDeep.get(global_state_1.globalState.batchDeep - 1).changeList.push({
            type: "action",
            action: debugOutputBundleAction
        });
    }
});
/**
 * debug 出栈 action
 */
global_state_1.globalState.event.on("endBatch", function () {
    if (!global_state_1.globalState.useDebug) {
        return;
    }
    // 每次出栈，都要更新 currentDebugOutputAction
    global_state_1.globalState.currentDebugOutputAction = global_state_1.globalState.debugOutputActionMapBatchDeep.get(global_state_1.globalState.batchDeep);
    if (!utils_1.inAction()) {
        var cloneDebugInfo = null;
        try {
            cloneDebugInfo = JSON.parse(JSON.stringify(global_state_1.globalState.debugOutputActionMapBatchDeep.get(1)));
        }
        catch (error) {
            return;
        }
        // 如果完全出队列了，把存储的 debug 信息输出给 debug 事件，并清空 debug 信息
        global_state_1.globalState.event.emit("debug", cloneDebugInfo);
        global_state_1.globalState.currentDebugOutputAction = null;
        global_state_1.globalState.debugOutputActionMapBatchDeep.clear();
        // 此时不能清空 globalState.currentDebugId = null，因为后续要传给 callback
        // action 与 debug 调用顺序：startBatch -> debugInAction -> ...multiple nested startBatch and endBatch -> debugOutAction -> reaction -> observe
    }
});
/**
 * 打印 diff 路径
 */
global_state_1.globalState.event.on("set", function (info) {
    if (!global_state_1.globalState.useDebug) {
        return;
    }
    var callStack = getCallStack(info.target);
    reportChange({
        type: "change",
        callStack: callStack,
        oldValue: cloneDeep(info.oldValue),
        key: info.key,
        value: cloneDeep(info.value)
    });
});
/**
 * 打印删除路径
 */
global_state_1.globalState.event.on("deleteProperty", function (info) {
    if (!global_state_1.globalState.useDebug) {
        return;
    }
    var callStack = getCallStack(info.target);
    reportChange({
        type: "delete",
        callStack: callStack,
        key: info.key,
    });
});
//# sourceMappingURL=debug.js.map