"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var event_1 = require("./event");
var tag = "ascoders-dob";
var globalOrWindow = (typeof self === "object" && self.self === self && self) ||
    (typeof global === "object" && global.global === global && global) ||
    this;
var GlobalState = (function () {
    function GlobalState() {
        /**
         * All proxies
         * key: proxies and it's origin object.
         */
        this.proxies = new WeakMap();
        /**
         * key: proxy
         * value: origin object
         */
        this.originObjects = new WeakMap();
        /**
         * Store all the original objects and keys to be proxied.
         */
        this.objectReactionBindings = new WeakMap();
        this.currentReaction = null;
        /**
         * Batch execution depth, such as each call runInAction in-depth +1, call -1, depth of 0 means that the implementation of the end.
         * When batchDeep == = 0, the operation queue is completed.
         */
        this.batchDeep = 0;
        this.pendingReactions = new Set();
        /**
         * Ignore the observable symbol
         */
        this.ignoreDynamicSymbol = Symbol();
        /**
         * Track Nested track, temporary cache down the track queue, waiting for the implementation of a track after the implementation.
         * Eg: observe(()=>{
         *  // ..
         *  observe(()=>{..})
         *  // ..
         * })
         * Special for nest react component.
         */
        this.pendingTracks = new Set();
        this.useDebug = false;
        /**
         * The currently executing debugOutputAction object.
         */
        this.currentDebugOutputAction = null;
        /**
         * The root of each batchDeep debugOutputBundleAction.
         */
        this.debugOutputActionMapBatchDeep = new Map();
        /**
         * Object's parent information, for debug.
         */
        this.parentInfo = new WeakMap();
        /**
         * Special for action name.
         */
        this.currentDebugName = null;
        this.currentDebugId = null;
        this.strictMode = false;
        this.event = new event_1.Event();
        this.uniqueIdCounter = 0;
    }
    return GlobalState;
}());
var globalState = new GlobalState();
exports.globalState = globalState;
if (globalOrWindow[tag]) {
    exports.globalState = globalState = globalOrWindow[tag];
}
else {
    globalOrWindow[tag] = globalState;
}
//# sourceMappingURL=global-state.js.map